﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

// Fanorona two-player game implementation
// kenrick@uaa.alaska.edu
// 10/2009
// 
// Use source at your own risk
// TODO: Proper documentation

namespace Fanornator
{
    public partial class Form1 : Form
    {
        private static int CELLSIZE = 50;
        private Board b = new Board();
        private int whoseturn = Board.WHITE;


        public Form1()
        {
            InitializeComponent();
        }

        private void pboxBoard_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            // Draw grid
            for (int x=0; x<8; x++)
                for (int y = 0; y < 4; y++)
                {
                    int xc = (x * CELLSIZE) + (CELLSIZE / 2);
                    int yc = (y * CELLSIZE) + (CELLSIZE / 2);

                    /*
                     * Test the connections made correctly by drawing the board
                     */
                    if (Board.isConnected(x,y,x+1,y+1))
                        g.DrawLine(Pens.Black, xc, yc, xc + CELLSIZE, yc + CELLSIZE);
                    if (Board.isConnected(x,y+1, x+1,y))
                        g.DrawLine(Pens.Black, xc, yc + CELLSIZE, xc + CELLSIZE, yc);
                    if (Board.isConnected(x,y, x+1,y))
                        g.DrawLine(Pens.Black, xc, yc, xc + CELLSIZE, yc);
                    if (Board.isConnected(x, y, x, y+1))
                        g.DrawLine(Pens.Black, xc, yc, xc, yc + CELLSIZE);
                    if (Board.isConnected(x, y+1, x + 1, y+1))
                        g.DrawLine(Pens.Black, xc, yc + CELLSIZE, xc + CELLSIZE, yc + CELLSIZE);
                    if (Board.isConnected(x+1, y+1, x + 1, y))
                        g.DrawLine(Pens.Black, xc + CELLSIZE, yc + CELLSIZE, xc + CELLSIZE, yc);

                    /*
                    if ((((x % 2) == 0) && ((y % 2) == 0))
                        ||
                         (((x % 2) == 1) && ((y % 2) == 1)))
                         g.DrawLine(Pens.Black, xc, yc, xc + CELLSIZE, yc + CELLSIZE);
                    else if ((((x % 2) == 0) && ((y % 2) == 1)) 
                            ||
                             (((x % 2) == 1) && ((y % 2) == 0)))
                        g.DrawLine(Pens.Black, xc, yc + CELLSIZE, xc + CELLSIZE, yc);
                    g.DrawLine(Pens.Black, xc, yc, xc + CELLSIZE, yc);
                    g.DrawLine(Pens.Black, xc, yc, xc, yc + CELLSIZE);
                    g.DrawLine(Pens.Black, xc, yc + CELLSIZE, xc + CELLSIZE, yc + CELLSIZE);
                    g.DrawLine(Pens.Black, xc + CELLSIZE, yc + CELLSIZE, xc + CELLSIZE, yc);
                     */
                }
            // Draw pieces
            for (int x = 0; x < 9; x++)
                for (int y = 0; y < 5; y++)
                {
                    int color = b.board[x, y];
                    int xc = (x * CELLSIZE) + (CELLSIZE / 2);
                    int yc = (y * CELLSIZE) + (CELLSIZE / 2);
                    if (color == Board.BLACK)
                    {
                        g.FillEllipse(Brushes.Black, xc - 10, yc - 10, 20, 20);
                    }
                    else if (color == Board.WHITE)
                    {
                        g.FillEllipse(Brushes.White, xc - 10, yc - 10, 20, 20);
                        g.DrawEllipse(Pens.Black, xc - 10, yc - 10, 20, 20);
                    }
                }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            txtMove.Focus();
        }

        private void button1_Click(object sender, EventArgs e)
        {


        }

        private void btnMove_Click(object sender, EventArgs e)
        {
            lblMessage.Text = "";
            string s = txtMove.Text.Trim().ToUpper();
            int x1, y1, x2, y2;
            char direction;
            direction = s[0];
            x1 = Convert.ToInt32(s[1] - '0');
            y1 = 4 - Convert.ToInt32(s[2] - '0');       // Convert to internal format
            x2 = Convert.ToInt32(s[3] - '0');
            y2 = 4 - Convert.ToInt32(s[4] - '0');       // COnvert to internal format
            if (direction == 'M')      // Single move, zugzwang
            {
                Move m = new Move(x1, y1, x2, y2);
                if (!b.ValidMove(m, whoseturn))
                {
                    lblMessage.Text = "Invalid Move.  Try again.";
                }
                else
                {
                    lstMoves.Items.Add(s);
                    lstMoves.SetSelected(lstMoves.Items.Count - 1, true);
                    b.MakeMove(m);
                    UpdateTurn();
                }
            }
            else
            {
                // List of captures separated by a space
                string[] arr = s.Split(" ".ToCharArray()); 
                Move m = new Move();
                foreach (string capture in arr)
                {
                    direction = capture[0];
                    // Translate to internal format
                    if (direction == 'C')
                        direction = 'A';                        
                    x1 = Convert.ToInt32(capture[1] - '0');
                    y1 = 4- Convert.ToInt32(capture[2] - '0');     // Convert
                    x2 = Convert.ToInt32(capture[3] - '0');
                    y2 = 4- Convert.ToInt32(capture[4] - '0');     // Convert
                    m.AddCapture(x1, y1, x2, y2, direction);
                }
                if (!b.ValidMove(m, whoseturn))
                {
                    lblMessage.Text = "Invalid Move.  Try again.";
                }
                else
                {
                    lstMoves.Items.Add(s);
                    lstMoves.SetSelected(lstMoves.Items.Count - 1, true);
                    b.MakeMove(m);
                    UpdateTurn();
                }
            }

            pboxBoard.Invalidate();
            txtMove.Text = "";
            txtMove.Focus();

            // Check for game over
            int result = b.gameOver();
            if (result == Board.BLACK)
            {
                lblMessage.Text = "BLACK WINS!";
                txtMove.Enabled = false;
                btnMove.Enabled = false;
            }
            else if (result == Board.WHITE)
            {
                lblMessage.Text = "WHITE WINS!";
                txtMove.Enabled = false;
                btnMove.Enabled = false;
            }
        }

        private void UpdateTurn()
        {
            whoseturn = b.OpponentColor(whoseturn);
            if (whoseturn == Board.BLACK)
                lblWhoseTurn.Text = "Black's Turn.";
            else
                lblWhoseTurn.Text = "White's Turn.";
        }

        private void txtMove_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13)
            {
                btnMove_Click(sender, e);
                e.Handled = true;
            }
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void helpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmHelp hlp = new frmHelp();
            hlp.ShowDialog();
        }
    }
}
